@@ -1,14 +1,18 @@
 import time
 import urllib
-from typing import Any, Dict, Optional
+from typing import TYPE_CHECKING
 
 from deprecated.sphinx import versionadded
 from packaging.version import Version
 
+from limits.aio.storage.base import MovingWindowSupport, Storage
 from limits.errors import ConfigurationError
+from limits.typing import AsyncRedisClient, Dict, Optional, Tuple, Union
 from limits.util import get_package_data
 
-from .base import MovingWindowSupport, Storage
+if TYPE_CHECKING:
+    import coredis
+    import coredis.commands.script
 
 
 class RedisInteractor:
@@ -21,14 +25,14 @@ class RedisInteractor:
     SCRIPT_CLEAR_KEYS = get_package_data(f"{RES_DIR}/clear_keys.lua")
     SCRIPT_INCR_EXPIRE = get_package_data(f"{RES_DIR}/incr_expire.lua")
 
-    lua_moving_window: Any
-    lua_acquire_window: Any
+    lua_moving_window: "coredis.commands.script.Script"
+    lua_acquire_window: "coredis.commands.script.Script"
 
     async def _incr(
         self,
         key: str,
         expiry: int,
-        connection,
+        connection: AsyncRedisClient,
         elastic_expiry: bool = False,
         amount: int = 1,
     ) -> int:
@@ -47,7 +51,7 @@ class RedisInteractor:
 
         return value
 
-    async def _get(self, key: str, connection) -> int:
+    async def _get(self, key: str, connection: AsyncRedisClient) -> int:
         """
         :param connection: Redis connection
         :param key: the key to get the counter value for
@@ -55,14 +59,16 @@ class RedisInteractor:
 
         return int(await connection.get(key) or 0)
 
-    async def _clear(self, key: str, connection) -> None:
+    async def _clear(self, key: str, connection: AsyncRedisClient) -> None:
         """
         :param key: the key to clear rate limits for
         :param connection: Redis connection
         """
         await connection.delete([key])
 
-    async def get_moving_window(self, key, limit, expiry):
+    async def get_moving_window(
+        self, key: str, limit: int, expiry: int
+    ) -> Tuple[int, int]:
         """
         returns the starting point and the number of entries in the moving
         window
@@ -79,7 +85,12 @@ class RedisInteractor:
         return window or (timestamp, 0)
 
     async def _acquire_entry(
-        self, key: str, limit: int, expiry: int, connection, amount: int = 1
+        self,
+        key: str,
+        limit: int,
+        expiry: int,
+        connection: AsyncRedisClient,
+        amount: int = 1,
     ) -> bool:
         """
         :param key: rate limit key to acquire an entry in
@@ -94,7 +105,7 @@ class RedisInteractor:
 
         return bool(acquired)
 
-    async def _get_expiry(self, key, connection=None):
+    async def _get_expiry(self, key: str, connection: AsyncRedisClient) -> int:
         """
         :param key: the key to get the expiry for
         :param connection: Redis connection
@@ -102,7 +113,7 @@ class RedisInteractor:
 
         return int(max(await connection.ttl(key), 0) + time.time())
 
-    async def _check(self, connection) -> bool:
+    async def _check(self, connection: AsyncRedisClient) -> bool:
         """
         check if storage is healthy
 
@@ -131,15 +142,22 @@ class RedisStorage(RedisInteractor, Storage, MovingWindowSupport):
     DEPENDENCIES = {"coredis": Version("3.4.0")}
 
     def __init__(
-        self, uri: str, connection_pool: Optional[Any] = None, **options
+        self,
+        uri: str,
+        connection_pool: Optional["coredis.ConnectionPool"] = None,
+        **options: Union[float, str, bool],
     ) -> None:
         """
-        :param uri: uri of the form `async+redis://[:password]@host:port`,
-         `async+redis://[:password]@host:port/db`,
-         `async+rediss://[:password]@host:port`, `async+unix:///path/to/sock` etc.
-         This uri is passed directly to :func:`coredis.Redis.from_url` with
-         the initial `a` removed, except for the case of `redis+unix` where it
-         is replaced with `unix`.
+        :param uri: uri of the form:
+
+         - ``async+redis://[:password]@host:port``
+         - ``async+redis://[:password]@host:port/db``
+         - ``async+rediss://[:password]@host:port``
+         - ``async+unix:///path/to/sock`` etc...
+
+         This uri is passed directly to :meth:`coredis.Redis.from_url` with
+         the initial ``async`` removed, except for the case of ``async+redis+unix``
+         where it is replaced with ``unix``.
         :param connection_pool: if provided, the redis client is initialized with
          the connection pool and any other params passed as :paramref:`options`
         :param options: all remaining keyword arguments are passed
@@ -149,9 +167,10 @@ class RedisStorage(RedisInteractor, Storage, MovingWindowSupport):
         uri = uri.replace("async+redis", "redis", 1)
         uri = uri.replace("redis+unix", "unix")
 
-        super().__init__()
+        super().__init__(uri, **options)
+
+        self.dependency = self.dependencies["coredis"].module
 
-        self.dependency = self.dependencies["coredis"]
         if connection_pool:
             self.storage = self.dependency.Redis(
                 connection_pool=connection_pool, **options
@@ -188,7 +207,7 @@ class RedisStorage(RedisInteractor, Storage, MovingWindowSupport):
                 key, expiry, self.storage, elastic_expiry, amount
             )
         else:
-            return await self.lua_incr_expire.execute([key], [expiry, amount])
+            return int(await self.lua_incr_expire.execute([key], [expiry, amount]))
 
     async def get(self, key: str) -> int:
         """
@@ -204,7 +223,9 @@ class RedisStorage(RedisInteractor, Storage, MovingWindowSupport):
 
         return await super()._clear(key, self.storage)
 
-    async def acquire_entry(self, key, limit, expiry, amount: int = 1) -> bool:
+    async def acquire_entry(
+        self, key: str, limit: int, expiry: int, amount: int = 1
+    ) -> bool:
         """
         :param key: rate limit key to acquire an entry in
         :param limit: amount of entries allowed
@@ -233,16 +254,12 @@ class RedisStorage(RedisInteractor, Storage, MovingWindowSupport):
         This function calls a Lua Script to delete keys prefixed with 'LIMITER'
         in block of 5000.
 
-        .. warning::
-           This operation was designed to be fast, but was not tested
+        .. warning:: This operation was designed to be fast, but was not tested
            on a large production based system. Be careful with its usage as it
            could be slow on very large data sets.
-
         """
 
-        cleared = await self.lua_clear_keys.execute(["LIMITER*"])
-
-        return cleared
+        return int(await self.lua_clear_keys.execute(["LIMITER*"]))
 
 
 @versionadded(version="2.1")
@@ -258,15 +275,15 @@ class RedisClusterStorage(RedisStorage):
     The storage schemes for redis cluster to be used in an async context
     """
 
-    DEFAULT_OPTIONS = {
+    DEFAULT_OPTIONS: Dict[str, Union[float, str, bool]] = {
         "max_connections": 1000,
     }
     "Default options passed to :class:`coredis.RedisCluster`"
 
-    def __init__(self, uri: str, **options):
+    def __init__(self, uri: str, **options: Union[float, str, bool]) -> None:
         """
         :param uri: url of the form
-         `async+redis+cluster://[:password]@host:port,host:port`
+         ``async+redis+cluster://[:password]@host:port,host:port``
         :param options: all remaining keyword arguments are passed
          directly to the constructor of :class:`coredis.RedisCluster`
         :raise ConfigurationError: when the coredis library is not
@@ -279,9 +296,11 @@ class RedisClusterStorage(RedisStorage):
             host, port = loc.split(":")
             cluster_hosts.append({"host": host, "port": int(port)})
 
-        super(RedisStorage, self).__init__()
-        self.dependency = self.dependencies["coredis"]
-        self.storage = self.dependency.RedisCluster(
+        super(RedisStorage, self).__init__(uri, **options)
+
+        self.dependency = self.dependencies["coredis"].module
+
+        self.storage: "coredis.RedisCluster[str]" = self.dependency.RedisCluster(
             startup_nodes=cluster_hosts, **{**self.DEFAULT_OPTIONS, **options}
         )
         self.initialize_storage(uri)
@@ -293,10 +312,10 @@ class RedisClusterStorage(RedisStorage):
         keys that are prefixed with 'LIMITER' and calls delete on them, one at
         a time.
 
-        .. warning::
-         This operation was not tested with extremely large data sets.
-         On a large production based system, care should be taken with its
-         usage as it could be slow on very large data sets"""
+        .. warning:: This operation was not tested with extremely large data sets.
+           On a large production based system, care should be taken with its
+           usage as it could be slow on very large data sets
+        """
 
         keys = await self.storage.keys("LIMITER*")
         return await self.storage.delete(keys)
@@ -313,7 +332,7 @@ class RedisSentinelStorage(RedisStorage):
     STORAGE_SCHEME = ["async+redis+sentinel"]
     """The storage scheme for redis accessed via a redis sentinel installation"""
 
-    DEFAULT_OPTIONS = {
+    DEFAULT_OPTIONS: Dict[str, Union[float, str, bool]] = {
         "stream_timeout": 0.2,
     }
     "Default options passed to :class:`~coredis.sentinel.Sentinel`"
@@ -323,13 +342,13 @@ class RedisSentinelStorage(RedisStorage):
     def __init__(
         self,
         uri: str,
-        service_name: str = None,
-        sentinel_kwargs: Optional[Dict[str, Any]] = None,
-        **options,
+        service_name: Optional[str] = None,
+        sentinel_kwargs: Optional[Dict[str, Union[float, str, bool]]] = None,
+        **options: Union[float, str, bool],
     ):
         """
         :param uri: url of the form
-         `async+redis+sentinel://host:port,host:port/service_name`
+         ``async+redis+sentinel://host:port,host:port/service_name``
         :param service_name, optional: sentinel service name
          (if not provided in `uri`)
         :param sentinel_kwargs, optional: kwargs to pass as
@@ -344,7 +363,7 @@ class RedisSentinelStorage(RedisStorage):
         sentinel_configuration = []
         connection_options = options.copy()
         sentinel_options = sentinel_kwargs.copy() if sentinel_kwargs else {}
-        parsed_auth = {}
+        parsed_auth: Dict[str, Union[float, str, bool]] = {}
 
         if parsed.username:
             parsed_auth["username"] = parsed.username
@@ -368,7 +387,7 @@ class RedisSentinelStorage(RedisStorage):
 
         super(RedisStorage, self).__init__()
 
-        self.dependency = self.dependencies["coredis.sentinel"]
+        self.dependency = self.dependencies["coredis.sentinel"].module
 
         self.sentinel = self.dependency.Sentinel(
             sentinel_configuration,
